import sys
import magic
import logging
import requests
from typing import Tuple
from pprint import pprint
from datetime import datetime
from bs4 import BeautifulSoup
from requests.models import Response
from urllib.parse import urljoin, urlparse

# Format the log message and append them into the logs list
def append_log(logs:list, log:str, url:str):
    '''
    Input: logs, log and url
    Output: append the message to the logs list.
    '''
    logs.append(url + " - ")
    logs.append(log + "\n")


# Write the examination resutls to log file. This is for demo purpose
# In practice, we use other methods, for example, email, to notify webmaster.
def write_to_log_file(logs:list, log_file:str = "C:\pyapps\monitor_landing_page\log_file.txt") -> bool:
    '''
    Input: logs, log_file
    Output: write the logs list into a physical file. returns true if succeed.
    '''
    rtn_value = True
    try:
        with open(log_file, "a") as log:
            # Append data to a file
            log.writelines(logs)
    except:
        rtn_value = False
    return rtn_value

# Use Beautiful Soap to parse the resposne content and examine images
def process_web_response(base_url:str, image_types:Tuple, response: Response, logs:list):
    '''
    Input: base_url, image_types, response, and logs
    Output: appends messages into the logs list
    '''
    # Create BeautifulSoup object
    soup = BeautifulSoup(response.content, 'html.parser')
    imgs = soup.find_all(lambda tag: tag.name == 'img' 
                                        and 'src' in tag.attrs
                                        and tag['src'] != '#')
    for img in imgs:
        img_src = img['src']
        fully_qualified_url = urljoin(base_url, img_src)
        try:
            req = requests.get(fully_qualified_url, stream = True)
            # Reference: https://pypi.org/project/python-magic/
            if (req.status_code == 200):
                p_buffer = req.raw.read(2048)
                result = magic.from_buffer(p_buffer)
                if not (result.startswith(image_types)):
                    msg = "Invalid image format: " + result.split(",")[0]
                    append_log(logs, msg, fully_qualified_url)
            else:
                msg = "HTTP status code: " + str(req.status_code)
                append_log(logs, msg, fully_qualified_url)
        except:
            msg = "Exception: " + str(sys.exc_info()[0])
            append_log(logs, msg, fully_qualified_url)
        finally:
            if req is not None:
                req.close()   

# Examine all article feature iamges
def check_web_page_images(base_url:str = 'https://www.mssqltips.com/', page_relative_url:str = '', image_types:Tuple = ('PNG', 'JPEG')):
    '''
    Input: base_url, page_relative_url and image_types
    Output: return a list of message, and also write these messages into a log file
    '''
    # Initializes variables
    logs = []
    req = None
    page_absolute_url = urljoin(base_url, page_relative_url)

    # Set logger.setLevel to DEBUG, the method prints all the debugs. 
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)

    # The starting time of execution of program
    start = datetime.now()
    logs.append("***Start***: " + start.strftime("%d/%m/%Y %H:%M:%S") + "\n")
    try:
        response = requests.get(page_absolute_url)
        if (response.status_code == 200):
            process_web_response(base_url, image_types, response, logs)
        else:
            msg = "HTTP status code: " + str(req.status_code)
            append_log(logs, msg, page_absolute_url)
    except:
        msg = "Exception: " + str(sys.exc_info()[0])
        append_log(logs, msg, page_absolute_url)
    finally:
        if response is not None:
            response.close()   
    # The ending time after execution of program
    end = datetime.now()
    logs.append("****End****: " + end.strftime("%d/%m/%Y %H:%M:%S") + "\n")
    # The time of execution of the program in between
    logs.append("The execution time: " + str(end-start)[5:] + " seconds \n")
    write_to_log_file(logs)
    return logs

# The value of the variable __name__ depends on how the code is being used. 
# A module's __name__ is set equal to __main__ when read from standard input, a script,
# or from an interactive prompt. 
if __name__ == '__main__':  # Use this condition to determine the execution conext.

    # Invoke this function to examine images on a web pages
    pprint(check_web_page_images())